@@ -69,11 +69,11 @@
 
 QT_BEGIN_NAMESPACE
 
-const quint64 FRAME_SIZE_IN_BYTES = 512 * 512 * 2;	//maximum size of a frame when sending a message
+const quint64 MAX_OUTGOING_FRAME_SIZE_IN_BYTES = std::numeric_limits<int>::max() - 1;
+const quint64 DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES = 512 * 512 * 2; //default size of a frame when sending a message
 
 QWebSocketConfiguration::QWebSocketConfiguration() :
 #ifndef QT_NO_SSL
-    m_sslConfiguration(QSslConfiguration::defaultConfiguration()),
     m_ignoredSslErrors(),
     m_ignoreSslErrors(false),
 #endif
@@ -107,12 +107,13 @@ QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::
     m_closeCode(QWebSocketProtocol::CloseCodeNormal),
     m_closeReason(),
     m_pingTimer(),
-    m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
     m_defaultMaskGenerator(),
-    m_handshakeState(NothingDoneState)
+    m_handshakeState(NothingDoneState),
+    m_outgoingFrameSize(DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES)
 {
+    m_pingTimer.start();
 }
 
 /*!
@@ -138,12 +139,13 @@ QWebSocketPrivate::QWebSocketPrivate(QTcpSocket *pTcpSocket, QWebSocketProtocol:
     m_closeCode(QWebSocketProtocol::CloseCodeNormal),
     m_closeReason(),
     m_pingTimer(),
-    m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
     m_defaultMaskGenerator(),
-    m_handshakeState(NothingDoneState)
+    m_handshakeState(NothingDoneState),
+    m_outgoingFrameSize(DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES)
 {
+    m_pingTimer.start();
 }
 
 /*!
@@ -154,6 +156,7 @@ void QWebSocketPrivate::init()
     Q_ASSERT(q_ptr);
     Q_ASSERT(m_pMaskGenerator);
 
+    m_dataProcessor->setParent(q_ptr);
     m_pMaskGenerator->seed();
 
     if (m_pSocket) {
@@ -386,7 +389,7 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
     }
     //if (m_url != url)
     if (Q_LIKELY(!m_pSocket)) {
-        m_dataProcessor.clear();
+        m_dataProcessor->clear();
         m_isClosingHandshakeReceived = false;
         m_isClosingHandshakeSent = false;
 
@@ -436,6 +439,7 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
                         sslSocket->ignoreSslErrors(m_configuration.m_ignoredSslErrors);
     #ifndef QT_NO_NETWORKPROXY
                     sslSocket->setProxy(m_configuration.m_proxy);
+                    m_pSocket->setProtocolTag(QStringLiteral("https"));
     #endif
                     sslSocket->connectToHostEncrypted(url.host(), quint16(url.port(443)));
                 } else {
@@ -458,6 +462,7 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
                 setSocketState(QAbstractSocket::ConnectingState);
     #ifndef QT_NO_NETWORKPROXY
                 m_pSocket->setProxy(m_configuration.m_proxy);
+                m_pSocket->setProtocolTag(QStringLiteral("http"));
     #endif
                 m_pSocket->connectToHost(url.host(), quint16(url.port(80)));
             } else {
@@ -565,15 +570,14 @@ void QWebSocketPrivate::enableMasking(bool enable)
 /*!
  * \internal
  */
-void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
+void QWebSocketPrivate::makeConnections(QTcpSocket *pTcpSocket)
 {
     Q_ASSERT(pTcpSocket);
     Q_Q(QWebSocket);
 
     if (Q_LIKELY(pTcpSocket)) {
         //pass through signals
-        QObject::connect(pTcpSocket,
-                         QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error),
+        QObject::connect(pTcpSocket, &QAbstractSocket::errorOccurred,
                          q, QOverload<QAbstractSocket::SocketError>::of(&QWebSocket::error));
 #ifndef QT_NO_NETWORKPROXY
         QObject::connect(pTcpSocket, &QAbstractSocket::proxyAuthenticationRequired, q,
@@ -616,22 +620,26 @@ void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
         }
     }
 
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::textFrameReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::textFrameReceived, q,
                      &QWebSocket::textFrameReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::binaryFrameReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::binaryFrameReceived, q,
                      &QWebSocket::binaryFrameReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::binaryMessageReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::binaryMessageReceived, q,
                      &QWebSocket::binaryMessageReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::textMessageReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::textMessageReceived, q,
                      &QWebSocket::textMessageReceived);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::errorEncountered, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::errorEncountered, this,
                             &QWebSocketPrivate::close);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::pingReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::pingReceived, this,
                             &QWebSocketPrivate::processPing);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::pongReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::pongReceived, this,
                             &QWebSocketPrivate::processPong);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::closeReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::closeReceived, this,
                             &QWebSocketPrivate::processClose);
+
+    //fire readyread, in case we already have data inside the tcpSocket
+    if (pTcpSocket->bytesAvailable())
+        Q_EMIT pTcpSocket->readyRead();
 }
 
 /*!
@@ -641,7 +649,7 @@ void QWebSocketPrivate::releaseConnections(const QTcpSocket *pTcpSocket)
 {
     if (Q_LIKELY(pTcpSocket))
         pTcpSocket->disconnect();
-    m_dataProcessor.disconnect();
+    m_dataProcessor->disconnect();
 }
 
 /*!
@@ -768,11 +776,11 @@ qint64 QWebSocketPrivate::doWriteFrames(const QByteArray &data, bool isBinary)
     const QWebSocketProtocol::OpCode firstOpCode = isBinary ?
                 QWebSocketProtocol::OpCodeBinary : QWebSocketProtocol::OpCodeText;
 
-    int numFrames = data.size() / int(FRAME_SIZE_IN_BYTES);
+    int numFrames = data.size() / int(outgoingFrameSize());
     QByteArray tmpData(data);
     tmpData.detach();
     char *payload = tmpData.data();
-    quint64 sizeLeft = quint64(data.size()) % FRAME_SIZE_IN_BYTES;
+    quint64 sizeLeft = quint64(data.size()) % outgoingFrameSize();
     if (Q_LIKELY(sizeLeft))
         ++numFrames;
 
@@ -791,7 +799,7 @@ qint64 QWebSocketPrivate::doWriteFrames(const QByteArray &data, bool isBinary)
         const bool isLastFrame = (i == (numFrames - 1));
         const bool isFirstFrame = (i == 0);
 
-        const quint64 size = qMin(bytesLeft, FRAME_SIZE_IN_BYTES);
+        const quint64 size = qMin(bytesLeft, outgoingFrameSize());
         const QWebSocketProtocol::OpCode opcode = isFirstFrame ? firstOpCode
                                                                : QWebSocketProtocol::OpCodeContinue;
 
@@ -965,7 +973,7 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
     case ReadingStatusState:
         if (!pSocket->canReadLine())
             return;
-        m_statusLine = pSocket->readLine();
+        m_statusLine = pSocket->readLine().trimmed();
         if (Q_UNLIKELY(!parseStatusLine(m_statusLine, &m_httpMajorVersion, &m_httpMinorVersion, &m_httpStatusCode, &m_httpStatusMessage))) {
             errorDescription = QWebSocket::tr("Invalid statusline in response: %1.").arg(QString::fromLatin1(m_statusLine));
             break;
@@ -996,8 +1004,8 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
                     errorDescription = QWebSocket::tr("Malformed header in response: %1.").arg(headerLine);
                     break;
                 }
-                lastHeader = m_headers.insertMulti(headerLine.left(colonPos).trimmed().toLower(),
-                                                   headerLine.mid(colonPos + 1).trimmed());
+                lastHeader = m_headers.insert(headerLine.left(colonPos).trimmed().toLower(),
+                                              headerLine.mid(colonPos + 1).trimmed());
             }
         }
 
@@ -1044,8 +1052,7 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
         } else if (m_httpStatusCode == 400) {
             //HTTP/1.1 400 Bad Request
             if (!version.isEmpty()) {
-                const QStringList versions = version.split(QStringLiteral(", "),
-                                                           QString::SkipEmptyParts);
+                const QStringList versions = version.split(QStringLiteral(", "), Qt::SkipEmptyParts);
                 if (!versions.contains(QString::number(QWebSocketProtocol::currentVersion()))) {
                     //if needed to switch protocol version, then we are finished here
                     //because we cannot handle other protocols than the RFC one (v13)
@@ -1095,9 +1102,6 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
     Q_ASSERT(m_pSocket);
     Q_Q(QWebSocket);
     QAbstractSocket::SocketState webSocketState = this->state();
-    int port = 80;
-    if (m_request.url().scheme() == QStringLiteral("wss"))
-        port = 443;
 
     switch (socketState) {
     case QAbstractSocket::ConnectedState:
@@ -1116,11 +1120,10 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
 
             const auto format = QUrl::RemoveScheme | QUrl::RemoveUserInfo
                                 | QUrl::RemovePath | QUrl::RemoveQuery
-                                | QUrl::RemoveFragment | QUrl::RemovePort;
+                                | QUrl::RemoveFragment;
             const QString host = m_request.url().toString(format).mid(2);
             const QString handshake = createHandShakeRequest(m_resourceName,
-                                                             host % QStringLiteral(":")
-                                                                  % QString::number(m_request.url().port(port)),
+                                                             host,
                                                              origin(),
                                                              QString(),
                                                              QString(),
@@ -1174,10 +1177,10 @@ void QWebSocketPrivate::processData()
     while (m_pSocket->bytesAvailable()) {
         if (state() == QAbstractSocket::ConnectingState) {
             if (!m_pSocket->canReadLine())
-                break;
+                return;
             processHandshake(m_pSocket);
-        } else {
-            m_dataProcessor.process(m_pSocket);
+        } else if (!m_dataProcessor->process(m_pSocket)) {
+            return;
         }
     }
 }
@@ -1301,6 +1304,80 @@ void QWebSocketPrivate::setSocketState(QAbstractSocket::SocketState state)
     }
 }
 
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setMaxAllowedIncomingFrameSize(quint64 maxAllowedIncomingFrameSize)
+{
+    m_dataProcessor->setMaxAllowedFrameSize(maxAllowedIncomingFrameSize);
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxAllowedIncomingFrameSize() const
+{
+    return m_dataProcessor->maxAllowedFrameSize();
+}
+
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setMaxAllowedIncomingMessageSize(quint64 maxAllowedIncomingMessageSize)
+{
+    m_dataProcessor->setMaxAllowedMessageSize(maxAllowedIncomingMessageSize);
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxAllowedIncomingMessageSize() const
+{
+    return m_dataProcessor->maxAllowedMessageSize();
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxIncomingMessageSize()
+{
+    return QWebSocketDataProcessor::maxMessageSize();
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxIncomingFrameSize()
+{
+    return QWebSocketDataProcessor::maxFrameSize();
+}
+
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setOutgoingFrameSize(quint64 outgoingFrameSize)
+{
+    if (outgoingFrameSize <= maxOutgoingFrameSize())
+        m_outgoingFrameSize = outgoingFrameSize;
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::outgoingFrameSize() const
+{
+    return m_outgoingFrameSize;
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxOutgoingFrameSize()
+{
+    return MAX_OUTGOING_FRAME_SIZE_IN_BYTES;
+}
+
+
 /*!
     \internal
  */
@@ -1449,6 +1526,7 @@ void QWebSocketPrivate::setReadBufferSize(qint64 size)
         m_pSocket->setReadBufferSize(m_readBufferSize);
 }
 
+#ifndef Q_OS_WASM
 /*!
     \internal
  */
@@ -1457,5 +1535,6 @@ bool QWebSocketPrivate::isValid() const
     return (m_pSocket && m_pSocket->isValid() &&
             (m_socketState == QAbstractSocket::ConnectedState));
 }
+#endif
 
 QT_END_NAMESPACE
